@@ -148,7 +148,7 @@ sub formbuilder (@) {
 			$f=Encode::decode_utf8($f);
 			$f=~s/^$page\///;
 			if (IkiWiki::isinlinableimage($f) &&
-			    UNIVERSAL::can("IkiWiki::Plugin::img", "import")) {
+			    IkiWiki::Plugin::img->can("import")) {
 				$add.='[[!img '.$f.' align="right" size="" alt=""]]';
 			}
 			else {
@@ -229,8 +229,10 @@ sub attachment_store {
 		check_canattach($session, $final_filename, $tempfile);
 	};
 	if ($@) {
-		json_response($q, $form, $dest."/".$filename, $@);
-		error $@;
+		# save error in case called functions clobber $@
+		my $error = $@;
+		json_response($q, $form, $dest."/".$filename, $error);
+		error $error;
 	}
 
 	# Move the attachment into holding directory.
@@ -274,19 +276,19 @@ sub attachments_save {
 	foreach my $filename (glob("$dir/*")) {
 		$filename=Encode::decode_utf8($filename);
 		next unless -f $filename;
-		my $destdir=$config{srcdir}."/".
-			linkpage(IkiWiki::possibly_foolish_untaint(
-				attachment_location($form->field('page'))));
+		my $destdir=linkpage(IkiWiki::possibly_foolish_untaint(
+			attachment_location($form->field('page'))));
+		my $absdestdir=$config{srcdir}."/".$destdir;
 		my $destfile=IkiWiki::basename($filename);
-		my $dest=$destdir.$destfile;
+		my $dest=$absdestdir.$destfile;
 		unlink($dest);
-		IkiWiki::prep_writefile($destfile, $destdir);
+		IkiWiki::prep_writefile($destfile, $absdestdir);
 		rename($filename, $dest);
-		push @attachments, $dest;
+		push @attachments, $destdir.$destfile;
 	}
 	return unless @attachments;
 	require IkiWiki::Render;
-	IkiWiki::prune($dir);
+	IkiWiki::prune($dir, $config{wikistatedir}."/attachments");
 
 	# Check the attachments in and trigger a wiki refresh.
 	if ($config{rcs}) {
