@@ -39,7 +39,7 @@ sub refresh () {
 				}
 				if ($delete) {
 					debug(sprintf(gettext("removing old preview %s"), $file));
-					IkiWiki::prune("$config{destdir}/$file");
+					IkiWiki::prune("$config{destdir}/$file", $config{destdir});
 				}
 			}
 			elsif (defined $mtime) {
@@ -64,7 +64,8 @@ sub cgi_editpage ($$) {
 
 	decode_cgi_utf8($q);
 
-	my @fields=qw(do rcsinfo subpage from page type editcontent editmessage);
+	my @fields=qw(do rcsinfo subpage from page type editcontent
+		editmessage subscribe);
 	my @buttons=("Save Page", "Preview", "Cancel");
 	eval q{use CGI::FormBuilder};
 	error($@) if $@;
@@ -157,6 +158,17 @@ sub cgi_editpage ($$) {
 			noimageinline => 1,
 			linktext => "FormattingHelp"));
 	
+	my $cansubscribe=IkiWiki::Plugin::notifyemail->can("subscribe")
+		&& IkiWiki::Plugin::comments->can("import")
+		&& defined $session->param('name');
+	if ($cansubscribe) {
+		$form->field(name => "subscribe", type => "checkbox",
+			options => [gettext("email comments to me")]);
+	}
+	else {
+		$form->field(name => "subscribe", type => 'hidden');
+	}
+	
 	my $previewing=0;
 	if ($form->submitted eq "Cancel") {
 		if ($form->field("do") eq "create" && defined $from) {
@@ -388,10 +400,12 @@ sub cgi_editpage ($$) {
 		eval { writefile($file, $config{srcdir}, $content) };
 		$config{cgi}=1;
 		if ($@) {
+			# save $@ in case a called function clobbers it
+			my $error = $@;
 			$form->field(name => "rcsinfo", value => rcs_prepedit($file),
 				force => 1);
 			my $mtemplate=template("editfailedsave.tmpl");
-			$mtemplate->param(error_message => $@);
+			$mtemplate->param(error_message => $error);
 			$form->tmpl_param("message", $mtemplate->output);
 			$form->field("editcontent", value => $content, force => 1);
 			$form->tmpl_param("page_select", 0);
@@ -448,6 +462,12 @@ sub cgi_editpage ($$) {
 			# caches and get the most recent version of the page.
 			redirect($q, $baseurl."?updated");
 		}
+
+		if ($cansubscribe && length $form->field("subscribe")) {
+			my $subspec="comment($page)";
+			IkiWiki::Plugin::notifyemail::subscribe(
+				$session->param('name'), $subspec);
+		}
 	}
 
 	exit;
