@@ -19,14 +19,14 @@ sub import {
 	hook(type => "checkconfig", id => "inline", call => \&checkconfig);
 	hook(type => "sessioncgi", id => "inline", call => \&sessioncgi);
 	hook(type => "preprocess", id => "inline", 
-		call => \&IkiWiki::preprocess_inline);
+		call => \&IkiWiki::preprocess_inline, scan => 1);
 	hook(type => "pagetemplate", id => "inline",
 		call => \&IkiWiki::pagetemplate_inline);
 	hook(type => "format", id => "inline", call => \&format, first => 1);
 	# Hook to change to do pinging since it's called late.
 	# This ensures each page only pings once and prevents slow
 	# pings interrupting page builds.
-	hook(type => "change", id => "inline", call => \&IkiWiki::pingurl);
+	hook(type => "rendered", id => "inline", call => \&IkiWiki::pingurl);
 }
 
 sub getopt () {
@@ -155,6 +155,23 @@ sub preprocess_inline (@) {
 	if (! exists $params{pages} && ! exists $params{pagenames}) {
 		error gettext("missing pages parameter");
 	}
+
+	if (! defined wantarray) {
+		# Running in scan mode: only do the essentials
+
+		if (yesno($params{trail}) && IkiWiki::Plugin::trail->can("preprocess_trailitems")) {
+			# default to sorting age, the same as inline itself,
+			# but let the params override that
+			IkiWiki::Plugin::trail::preprocess_trailitems(sort => 'age', %params);
+		}
+
+		return;
+	}
+
+	if (yesno($params{trail}) && IkiWiki::Plugin::trail->can("preprocess_trailitems")) {
+		scalar IkiWiki::Plugin::trail::preprocess_trailitems(sort => 'age', %params);
+	}
+
 	my $raw=yesno($params{raw});
 	my $archive=yesno($params{archive});
 	my $rss=(($config{rss} || $config{allowrss}) && exists $params{rss}) ? yesno($params{rss}) : $config{rss};
@@ -169,6 +186,13 @@ sub preprocess_inline (@) {
 	if (! exists $params{feedshow} && exists $params{show}) {
 		$params{feedshow}=$params{show};
 	}
+	my $title;
+	if (exists $params{title}) {
+		$title = $params{title};
+	}
+	else {
+		$title = $params{page} ne "index" ? pagetitle($params{page}) : $config{wikiname};
+	}
 	my $desc;
 	if (exists $params{description}) {
 		$desc = $params{description} 
@@ -194,8 +218,7 @@ sub preprocess_inline (@) {
 			}
 		}
 
-		@list = map { bestlink($params{page}, $_) }
-		        split ' ', $params{pagenames};
+		@list = split ' ', $params{pagenames};
 
 		if (yesno($params{reverse})) {
 			@list=reverse(@list);
@@ -204,6 +227,8 @@ sub preprocess_inline (@) {
 		foreach my $p (@list) {
 			add_depends($params{page}, $p, deptype($quick ? "presence" : "content"));
 		}
+
+		@list = grep { exists $pagesources{$_} } @list;
 	}
 	else {
 		my $num=0;
@@ -366,7 +391,9 @@ sub preprocess_inline (@) {
 					blind_cache => 1);
 			};
 			if ($@) {
-				error sprintf(gettext("failed to process template %s"), $params{template}.".tmpl").": $@";
+				# gettext can clobber $@
+				my $error = $@;
+				error sprintf(gettext("failed to process template %s"), $params{template}.".tmpl").": $error";
 			}
 		}
 		my $needcontent=$raw || (!($archive && $quick) && $template->query(name => 'content'));
@@ -447,7 +474,7 @@ sub preprocess_inline (@) {
 			if (! $params{preview}) {
 				writefile($rssp, $config{destdir},
 					genfeed("rss",
-						$config{url}."/".$rssp, $desc, $params{guid}, $params{page}, @feedlist));
+						$config{url}."/".$rssp, $title, $desc, $params{guid}, $params{page}, @feedlist));
 				$toping{$params{destpage}}=1 unless $config{rebuild};
 				$feedlinks{$params{destpage}}.=qq{<link rel="alternate" type="application/rss+xml" title="$rssdesc" href="$rssurl" />};
 			}
@@ -457,7 +484,7 @@ sub preprocess_inline (@) {
 			will_render($params{destpage}, $atomp);
 			if (! $params{preview}) {
 				writefile($atomp, $config{destdir},
-					genfeed("atom", $config{url}."/".$atomp, $desc, $params{guid}, $params{page}, @feedlist));
+					genfeed("atom", $config{url}."/".$atomp, $title, $desc, $params{guid}, $params{page}, @feedlist));
 				$toping{$params{destpage}}=1 unless $config{rebuild};
 				$feedlinks{$params{destpage}}.=qq{<link rel="alternate" type="application/atom+xml" title="$atomdesc" href="$atomurl" />};
 			}
@@ -506,7 +533,7 @@ sub get_inline_content ($$) {
 		if (isinternal($page)) {
 			# make inlined text of internal pages searchable
 			run_hooks(indexhtml => sub {
-				shift->(page => $page, destpage => $page,
+				shift->(page => $page, destpage => $destpage,
 					content => $ret);
 			});
 		}
@@ -593,9 +620,30 @@ sub absolute_urls ($$) {
 	return $ret;
 }
 
+sub genenclosure {
+	my $itemtemplate=shift;
+	my $url=shift;
+	my $file=shift;
+
+	return unless $itemtemplate->query(name => "enclosure");
+
+	my $size=(srcfile_stat($file))[8];
+	my $mime="unknown";
+	eval q{use File::MimeInfo};
+	if (! $@) {
+		$mime = mimetype($file);
+	}
+	$itemtemplate->param(
+		enclosure => $url,
+		type => $mime,
+		length => $size,
+	);
+}
+
 sub genfeed ($$$$$@) {
 	my $feedtype=shift;
 	my $feedurl=shift;
+        my $feedtitle=shift;
 	my $feeddesc=shift;
 	my $guid=shift;
 	my $page=shift;
@@ -609,6 +657,7 @@ sub genfeed ($$$$$@) {
 	foreach my $p (@pages) {
 		my $u=URI->new(encode_utf8(urlto($p, "", 1)));
 		my $pcontent = absolute_urls(get_inline_content($p, $page), $url);
+		my $fancy_enclosure_seen = 0;
 
 		$itemtemplate->param(
 			title => pagetitle(basename($p)),
@@ -630,32 +679,23 @@ sub genfeed ($$$$$@) {
 				$itemtemplate->param(mdate_822 => date_822($pagestate{$p}{meta}{updated}));
 				$itemtemplate->param(mdate_3339 => date_3339($pagestate{$p}{meta}{updated}));
 			}
+
+			if (exists $pagestate{$p}{meta}{enclosure}) {
+				my $absurl = $pagestate{$p}{meta}{enclosure};
+				my $file = $pagestate{$p}{meta}{enclosurefile};
+				genenclosure($itemtemplate, $absurl, $file);
+				$fancy_enclosure_seen = 1;
+			}
 		}
 
-		if ($itemtemplate->query(name => "enclosure")) {
-			my $file=$pagesources{$p};
-			my $type=pagetype($file);
-			if (defined $type) {
-				$itemtemplate->param(content => $pcontent);
-			}
-			else {
-				my $size=(srcfile_stat($file))[8];
-				my $mime="unknown";
-				eval q{use File::MimeInfo};
-				if (! $@) {
-					$mime = mimetype($file);
-				}
-				$itemtemplate->param(
-					enclosure => $u,
-					type => $mime,
-					length => $size,
-				);
-			}
-		}
-		else {
-			$itemtemplate->param(content => $pcontent);
+		my $file=$pagesources{$p};
+		unless ($fancy_enclosure_seen || defined(pagetype($file))) {
+			genenclosure($itemtemplate, $u, $file);
+			$itemtemplate->param(simplepodcast => 1);
 		}
 
+		$itemtemplate->param(content => $pcontent);
+
 		run_hooks(pagetemplate => sub {
 			shift->(page => $p, destpage => $page,
 				template => $itemtemplate);
@@ -669,15 +709,15 @@ sub genfeed ($$$$$@) {
 
 	my $template=template_depends($feedtype."page.tmpl", $page, blind_cache => 1);
 	$template->param(
-		title => $page ne "index" ? pagetitle($page) : $config{wikiname},
+		title => $feedtitle,
 		wikiname => $config{wikiname},
 		pageurl => $url,
 		content => $content,
 		feeddesc => $feeddesc,
 		guid => $guid,
 		feeddate => date_3339($lasttime),
+		feeddate_822 => date_822($lasttime),
 		feedurl => $feedurl,
-		version => $IkiWiki::version,
 	);
 	run_hooks(pagetemplate => sub {
 		shift->(page => $page, destpage => $page,
